pub enum CryptoAlgorithm {
XC20P,
A256GCM,
A256CBC,
}Expand description
Pluggable closure generator enum, which creates instance of crypto function based on selected algorithm types.
Attention:
Immutable by design and should be instance per invocation to make sure no sensitive data is been stored in memory longer than necessary. Underlying algorithms are implemented by Rust-crypto crate family.
Allowed (and implemented) cryptographical algorithms (JWA). According to spec
Variants