AttachmentDataBuilder

Struct AttachmentDataBuilder 

pub struct AttachmentDataBuilder { /* private fields */ }
Builder for AttachmentData

Implementations

Source

impl AttachmentDataBuilder

pub fn new() -> Self

Constructor for default and empty data

pub fn with_jws(self, jws: &str) -> Self

Attach jws stringified property.

Parameters
  • jws - JSON Web Signature serialized into String
pub fn with_hash(self, hash: &str) -> Self

[optional] The hash of the content encoded in multi-hash format. Used as an integrity check for the attachment, and MUST be used if the data is referenced via the links data attribute.

Parameters
  • hash - String of hash to be attached

[optional] A list of zero or more locations at which the content may be fetched. Adds one link into list of links. No uniqueness is guarranteed.

Parameters
  • link - String representation of where to fetch the attachment
pub fn with_raw_payload(self, payload: impl AsRef<[u8]>) -> Self

Raw bytes of the payload to be attached - will be BASE64URL encoded before attaching.

Parameters
  • payload - set of bytes to be attached as payload
pub fn with_encoded_payload(self, payload: &str) -> Self

Same as with_raw_payload, but data is already encoded

Parameters
  • payload - BASE64URL encoded bytes of payload
pub fn with_json(self, stringified: &str) -> Self

Attach stringified JSON object

Parameters
  • stringified - String of JSON object