Problem

Struct Problem 

pub struct Problem { /* private fields */ }

Implementations

Source

impl Problem

pub fn from_code(code: &str) -> Result<Self, Error>

Parses given code into new instance of Problem If given string does not match regex “” - returns Err.

pub fn from_known_problem(problem: KnownProblems) -> Self

Takes KnownProblem variant and creates new Problem using it as code. Kwnown problems are defined in the specifications of the didcomm v2.