pub struct JwmHeader {
pub typ: MessageType,
pub enc: Option<String>,
pub kid: Option<String>,
pub skid: Option<String>,
pub alg: Option<String>,
pub jku: Option<String>,
pub jwk: Option<Jwk>,
pub epk: Option<Jwk>,
pub cty: Option<String>,
}Expand description
JWM Header as specified in RFC With single deviation - allows raw text JWM to support DIDComm spec
Designed to work for both JWE and JWS message types.
iv property is not explicitly listed in the registered properties on the RFC but is present
within example lists - used here as DIDComm crypto nonce sharing property.
Fields
typ: MessageTypeenc: Option<String>kid: Option<String>skid: Option<String>alg: Option<String>jku: Option<String>jwk: Option<Jwk>epk: Option<Jwk>cty: Option<String>Implementations
Sourceimpl JwmHeader
impl JwmHeader
Sourcepub fn as_signed(&mut self, alg: &SignatureAlgorithm)
pub fn as_signed(&mut self, alg: &SignatureAlgorithm)
Setter of JOSE header properties to identify which signature alg used.
Modifies typ and alg headers.
Sourcepub fn as_encrypted(&mut self, alg: &CryptoAlgorithm)
pub fn as_encrypted(&mut self, alg: &CryptoAlgorithm)
Setter of JOSE header properties to identify which crypto alg and key type used.
Modifies enc, typ and alg headers.
pub fn kid(&mut self, kid: Option<String>)